<?php
/**
 * Pipelines du plugin Incarner
 *
 * @plugin     Incarner
 * @copyright  2016
 * @author     Michel Bystranowski
 * @licence    GNU/GPL
 */

/**
 * Determiner si on affiche le login ou l'email
 * @param $login
 * @param $email
 * @return mixed
 */
function incarner_login_affiche($login, $email) {
	$login_aff = $login ? $login : $email;
	// si le login est un md5 et qu'on a un email, afficher l'email
	if (strlen($login_aff) == 32
		and !preg_match(",[^0-9a-f],i", $login_aff)
	  and $email) {
		$login_aff = $email;
	}
	return $login_aff;
}

/**
 * Afficher un lien pour incarner un auteur sur sa page
 *
 * @pipeline affiche_gauche
 * @param  array $flux Données du pipeline
 * @return array       Données du pipeline
 */
function incarner_boite_infos($flux) {

	if (($flux['args']['type'] === 'auteur')
			and (autoriser('incarner'))) {
		include_spip('base/abstract_sql');
		include_spip('inc/session');

		$id_auteur = $flux['args']['id'];

		if ($id_auteur != session_get('id_auteur')) {
			$auteur = sql_fetsel(
				'login,email',
				'spip_auteurs',
				'id_auteur=' . intval($id_auteur)
			);
			$login_aff = incarner_login_affiche($auteur['login'], $auteur['email']);
			if ($login_aff) {
				if ($auteur_login_possible = auth_identifier_login($auteur['login'], '')){
					$url_self = urlencode(self());
					$url_action = generer_url_action(
						'incarner',
						'login=' . $auteur['login'] . '&redirect=' . $url_self
					);
					$disabled = "";
				}
				else {
					$url_action = "#";
					$disabled = "disabled";
				}

				$contexte = array(
					'url'     => $url_action,
					'texte'   => _T('incarner:incarner_login', array('login' => $login_aff)),
					'disable' => $disabled,
				);
				$fond_previsu = recuperer_fond('prive/squelettes/inclure/inc-incarner_bouton', $contexte);
				$flux['data'] .= $fond_previsu;
			}
		}
	}

	return $flux;
}

/**
 * Ajouter un lien dans côté public pour redevenir webmestre
 *
 * @pipeline formulaire_admin
 * @param  array $html Données du pipeline
 * @return array       Données du pipeline
 */
function incarner_affichage_final($html) {

	$id_auteur = incarner_racine_incarnation ();
	if (!$id_auteur) {
		return $html;
	}

	include_spip('base/abstract_sql');
	$auteur = sql_fetsel(
		'login,email',
		'spip_auteurs',
		'id_auteur=' . intval($id_auteur)
	);

	$self = urlencode(self());
	$url = generer_url_action(
		'incarner',
		'login=' . $auteur['login'] . '&redirect=' . $self
	);

	$url_logout = generer_url_action(
		'incarner',
		'logout=oui&redirect=' . $self
	);
	$login_aff = incarner_login_affiche($auteur['login'], $auteur['email']);
	$quisuisje = incarner_login_affiche(session_get('login'), session_get('email'));

	$lien = '<div class="menu-incarner' . (test_espace_prive() ? ' prive' : '') . '">';
	$lien .= '<a class="bouton-incarner" href="' . $url_logout . '">';
	$lien .= _T('incarner:logout_definitif');
	$lien .= '</a>';
	$invite = _T('incarner:reset_incarner', array('login' => $login_aff));
	$lien .= '<a class="bouton-incarner" href="' . $url . '" title="' . attribut_html($quisuisje.' : '.$invite) . '">';
	$lien .= $invite;
	$lien .= '</a>';
	$lien .= '</div>';

	$html = preg_replace('#(</body>)#', $lien . '$1', $html);
	return $html;
}

/**
 * teste si l'internaute actuellement connecté est un admin incarné
 *
 * @return int      id_auteur de l'admin actuellement incarné, ou 0 sinon
 */
function incarner_racine_incarnation(){
	if (
		(! isset($_COOKIE['spip_cle_incarner']))
		or (! $cle_actuelle = $_COOKIE['spip_cle_incarner'])
	) {
		return 0;
	}

	include_spip('inc/config');
	if (! ($cles = lire_config('incarner/cles'))) {
		return 0;
	}
	$id_auteur = array_search($cle_actuelle, $cles);

	include_spip('inc/session');
	include_spip('incarner_fonctions');
	if (
		incarner_cle_valide($cle_actuelle)
		and (intval(session_get('id_auteur')) !== $id_auteur)
	) {
		return $id_auteur;
	}
	return 0;
}

/**
 * @param $html
 * @return array
 */
function incarner_affichage_final_prive($html) {

	return incarner_affichage_final($html);
}

/**
 * Ajoute une feuille de styles à l'espace public
 *
 * @pipeline insert_head
 * @param  string $flux Données du pipeline
 * @return string       Données du pipeline
 */
function incarner_insert_head($flux) {

	$flux .= '<link rel="stylesheet" type="text/css" href="' . find_in_path('css/incarner.css'). '" />';

	return $flux;
}

function incarner_header_prive($flux) {

	return incarner_insert_head($flux);
}

/**
 * Donner un cookie d'incarnation aux webmestres dès le login
 *
 * @pipeline formulaire_traiter
 * @param  array $flux Données du pipeline
 * @return array       Données du pipeline
 */
function incarner_formulaire_traiter($flux) {

	if (($flux['args']['form'] === 'login') and autoriser('webmestre')) {
		incarner_renouveler_cle();
	}

	return $flux;
}

/**
 * Ajoute le bouton d'administration permettant d'incarner l'auteur de l'objet courant sur une page publique
 *
 * @pipeline formulaire_admin
 * @param array $flux Données du pipeline
 * @return array       Données du pipeline
 *
 * @uses _INCARNER_OBJET_ID_OBJET_COURANT
 *      Au format "objet|id_objet" = indique de quels objets on peut incarner l'auteur
 */
function incarner_formulaire_admin($flux) {

	if (!defined('_INCARNER_OBJET_ID_OBJET_COURANT')
		or empty($GLOBALS['visiteur_session']['id_auteur']) // utile lorsqu'on déconnecté mais que le cookie de correspondance est encore là
		or !isset($flux['args']['contexte']['objet'])
		or  !($objet = $flux['args']['contexte']['objet'])
		or !isset($flux['args']['contexte']['id_objet'])
		or !($id_objet = $flux['args']['contexte']['id_objet'])
		or !autoriser('incarner')
	) {
		return $flux;
	}
	list ($incarner_objet, $incarner_objet_id) = explode('|', _INCARNER_OBJET_ID_OBJET_COURANT);
	$id_objet_courant = intval(_request($incarner_objet_id));
	if ($objet != $incarner_objet) {
		return $flux;
	}

	include_spip('base/abstract_sql');
	$login_objet_courant = sql_getfetsel(
		'login',
		'spip_auteurs, spip_auteurs_liens',
		array(
			'spip_auteurs.id_auteur=spip_auteurs_liens.id_auteur',
			"objet='$objet'",
			"id_objet=$id_objet_courant"
		)
	);

	if (!$login_objet_courant) {
		return $flux;
	}
	$login = session_get('login');
	if ($login == $login_objet_courant){
		return $flux;
	}

	$url_courant = generer_url_action(
		'incarner',
		'login=' . $login_objet_courant . '&redirect=' . urlencode(self())
	);
	$lien = '<a class="bouton-incarner" href="' . $url_courant . '">';
	$lien .= _T ('incarner:incarner_login', array('login' => $login_objet_courant));
	$lien .= '</a>';

	$x='<!--extra-->';
	$flux['data'] = str_ireplace($x, $lien.$x, $flux['data']);
	return $flux;
}